#include <windows.h>
#include <strsafe.h>
#include "resource.h"
#include "plugin.h"


static HINSTANCE hInstance = NULL;
static POINT position = {0, 0};

typedef struct PREFS {
  unsigned int enabled;
  unsigned int num0;
  unsigned int dirnum;
  unsigned int filesub;
} PREFS;
static PREFS myPrefs;


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd );
#ifdef __cplusplus
extern "C"{
#endif
BOOL WINAPI _DllMainCRTStartup( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  return DllMain(hModule, reason, resvd);
}
#ifdef __cplusplus
};
#endif


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
  return TRUE;
}


static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2), rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

static void PositionWindow( HWND hdlg, POINT *pt)
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}

EXPORT int FileFilterInitialize( MINIFILEPARAM *param )
{
  DWORD type, len;
  PREFS *prefs;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  StringCchCopy(param->title, MAXTITLE, "File Numberer");
  param->version = PLUGINVERSION;
  param->user = &myPrefs;
  prefs = (PREFS*) param->user;
  prefs->enabled = FALSE;
  prefs->filesub = FALSE;
  prefs->dirnum = FALSE;
  prefs->num0 = FALSE;

  len = sizeof(PREFS);
  type = REG_BINARY;
  RegQueryValueEx(param->userKey, "prefs", 0, &type,
		  (unsigned char *)param->user, &len);

  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "position", 0, &type,
		  (unsigned char *)&position, &len);

  return TRUE;
}


static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIFILEPARAM *param;
  static PREFS *prefs;
  int enabled;

  switch(msg) {
  case WM_USER:
    enabled = (SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)==
	       BST_CHECKED)?TRUE:FALSE;
    EnableWindow(GetDlgItem(hDlg, IDC_NUM0), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_DIRNUM), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_FILESUB), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_INFO), enabled);
    return TRUE;

  case WM_INITDIALOG:
    param = (MINIFILEPARAM*)lParam;
    prefs = (PREFS*)param->user;
    PositionWindow(hDlg, &position);
    SendDlgItemMessage(hDlg, IDC_ENABLE, BM_SETCHECK,
		       prefs->enabled?BST_CHECKED:BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_DIRNUM, BM_SETCHECK,
		       prefs->dirnum?BST_CHECKED:BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_NUM0, BM_SETCHECK,
		       prefs->num0?BST_CHECKED:BST_UNCHECKED, 0);
    SendDlgItemMessage(hDlg, IDC_FILESUB, BM_SETCHECK,
		       prefs->filesub?BST_CHECKED:BST_UNCHECKED, 0);
    SetFocus(GetDlgItem(hDlg, IDC_ENABLE));
    SendMessage(hDlg, WM_USER, 0, 0);
    return FALSE;


  case WM_COMMAND:
    switch (LOWORD(wParam)) {
    case IDOK:
      SaveWindowPosition(hDlg, &position);
      prefs->enabled = SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)
	==BST_CHECKED?TRUE:FALSE;
      prefs->dirnum = SendDlgItemMessage(hDlg, IDC_DIRNUM, BM_GETCHECK, 0, 0)
	==BST_CHECKED?TRUE:FALSE;
      prefs->num0 = SendDlgItemMessage(hDlg, IDC_NUM0, BM_GETCHECK, 0, 0)
	==BST_CHECKED?TRUE:FALSE;
      prefs->filesub = SendDlgItemMessage(hDlg, IDC_FILESUB, BM_GETCHECK, 0, 0)
	==BST_CHECKED?TRUE:FALSE;
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position);
      EndDialog(hDlg, FALSE);
      return TRUE;

    case IDC_ENABLE:
      return SendMessage(hDlg, WM_USER, 0, 0);
		
    }
    break;
  }
  return FALSE;
}



EXPORT void FileFilterUserInterface( MINIFILEPARAM *param, HWND parent )
{
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent,
		 Dialog, (LPARAM)param);
}

void NameZipFiles(MINIZIPINFO *mz, PREFS *prefs, int txtCnt, int fileCnt,
		  int *oTxtCnt, int *oFileCnt)
{
  char buff[NAMELEN*2];
  unsigned int i;

  for (i=0; i<mz->numfiles; i++) {
    if (mz->fi[i].isdir) {
      if (prefs->dirnum) {
				wsprintf(buff, "%d. %s", txtCnt++, mz->fi[i].name);
				StringCchCopy(mz->fi[i].name, NAMELEN, buff);
      }
      mz->fi[i].size = fileCnt++;
      if (prefs->filesub)
				NameZipFiles(mz->fi[i].zi, prefs, prefs->num0?0:1, fileCnt,
										 NULL, &fileCnt);
      else
				NameZipFiles(mz->fi[i].zi, prefs, txtCnt, fileCnt, &txtCnt, &fileCnt);
    } else {
      wsprintf(buff, "%d. %s", txtCnt++, mz->fi[i].name);
      StringCchCopy(mz->fi[i].name, NAMELEN, buff);
      mz->fi[i].size = fileCnt++;
    }
  }

  if (oTxtCnt) *oTxtCnt=txtCnt;
  if (oFileCnt) *oFileCnt=fileCnt;
}

EXPORT MINIZIPINFO *FileFilterExecute( MINIFILEPARAM *param,
				       MINIZIPINFO *mz,
				       char *label,
				       unsigned int labelSize,
				       char *path )
{
  PREFS *prefs;

  prefs = (PREFS*)param->user;

  if (!prefs->enabled) return mz;
	
  NameZipFiles(mz, prefs, prefs->num0?0:1, 1, NULL, NULL);

  return mz;
}

EXPORT void FileFilterUninitialize( MINIFILEPARAM *param )
{
  RegSetValueEx(param->userKey, "position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
  RegSetValueEx(param->userKey, "prefs", 0, REG_BINARY,
		(unsigned char*)param->user, sizeof(PREFS));
}


